; SketchyLISP Library
; Copyright (C) 2005 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; count

; ---conformance---
; Sketchy Extension

; ---purpose---
; Recursively count the atoms of a list.
; Members of all sublists are included.

; ---args---
; X - list

; ---keywords---
; COUNT function, list, member, size

; ---see-also---
; length, depth, append

; ---example---
; (count '(a b (c (d) e) ((f)))) => 6

(define count #t)

:c lib/nplus.l n+

; ---model---
; (define count (lambda (x)
;	(cond ((null? x) 0)
;		((pair? x) (n+ (count (car x)) (count (cdr x))))
;		(#t 1))))

; ---code---
(define count (lambda (x)
	(letrec ((cnt (lambda (x r s)
			(cond ((null? x)
					(cond ((null? s) r)
						(#t (cnt (car s) r (cdr s)))))
				((pair? x)
					(cnt (car x) r (cons (cdr x) s)))
				(#t
					(cnt '() (n+ r 1) s))))))
		(cnt x 0 '()))))

