; SketchyLISP Library
; Copyright (C) 2005 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; expt

; ---conformance---
; R5RS

; ---purpose---
; Compute the X raised to the power of Y.
; X may be a natural or integer number.
; Y must be natural.

; ---args---
; X - number (base)
; Y - number (exponent)

; ---keywords---
; EXPT function, base, exponent, power

; ---see-also---
; digits, sqrt, gcd

; ---example---
; (expt 2 5) => 32

(define expt #t)

:c lib/digits.l
:c lib/negativep.l negative?
:c lib/abs.l
:c lib/zerop.l zero?
:c lib/evenp.l even?
:c lib/times.l *
:c lib/divide.l
:c lib/nquotient.l
:c lib/quotient.l
:c lib/integer.l

; ---model---
; (define expt (lambda (x y)
;	(letrec ((exp (lambda (x y r)
;			(cond ((zero? y) r)
;				(#t (exp x (- y 1) (* x r)))))))
;		(exp (integer x) (natural y) 1))))

; ---code---
(define expt (lambda (x y)
	(letrec
		((square (lambda (x)
			(* x x)))
		(exp (lambda (x y)
			(cond ((zero? y) 1)
				((even? y)
					(square (exp x (nquotient y 2))))
				(#t (* x (square (exp x (nquotient y 2)))))))))
		(exp (integer x) (natural y)))))

