; SketchyLISP Library
; Copyright (C) 2005 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; map

; ---conformance---
; R5RS

; ---purpose---
; Map N lists to one using an N-ary function F.
; F is applied to the the list consisting of the
; car of each argument list, giving the first
; member of the result list. MAP then proceeds
; with the cadr parts of each list, giving the
; next member of the result list, etc.

; ---args---
; F - function to apply
; A... - lists

; ---keywords---
; MAP function, mapping, lists, CAR

; ---see-also---
; member, list?

; ---example---
; :l lib/minus.l
; (map cons '(a b c) '(d e f)) => '((a.d) (b.e) (c.f))

(define map #t)

:c lib/reverse.l
:c lib/caar.l cdar

; ---code---
(define map (lambda (f . a)
	(letrec ((carof (lambda (a b)
				(cond ((null? a) (reverse b))
					(#t (carof (cdr a) (cons (caar a) b))))))
		(cdrof (lambda (a b)
				(cond ((null? a) (reverse b))
					(#t (cdrof (cdr a) (cons (cdar a) b))))))
		(mapc (lambda (a b)
			(cond ((null? (car a)) (reverse b))
				(#t (mapc (cdrof a '())
						(cons (apply f (carof a '())) b)))))))
		(cond ((null? a) (bottom '(too few arguments to map)))
			(#t (mapc a '()))))))

