; SketchyLISP Library
; Copyright (C) 2005 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; n-

; ---conformance---
; Sketchy Core

; ---purpose---
; Subtract two natural numbers.

; ---args---
; A - natural number
; B - natural number to be subtracted

; ---keywords---
; N- function, difference, subtract, subtraction
; arithmetics, natural

; ---see-also---
; digits, n+, nquotient, nremainder, n*, -

; ---example---
; (n- 7 5) => 2

(define nminus #t)

:c lib/digits.l 0-digit 1-digit 9-digit
:c lib/append.l
:c lib/reverse.l
:c lib/pred.l
:c lib/normalize.l

; ---code---
(define n- (lambda (a b)
	(letrec

		; Compute difference of two digits with borrow
		; X Y - digits
		; CI CO - borrow (carry) in, out
		; RESULT - (difference . carry out)
		((ddiff (lambda (x y ci co)
			(cond ((eq? y 0-digit)
					(cond ((eq? ci 1-digit)
							; subtract carry in
							(ddiff x 1-digit 0-digit co))
						(#t (cons x co))))
				; handle overflow
				((null? (pred x))
					(ddiff 9-digit (pred y) ci 1-digit))
				(#t (ddiff (pred x) (pred y) ci co)))))

		; Subtract two lists of decimal digits forming reverse
		; decimal numbers. The result is in normal order.
		; A B - reverse lists of digits
		; C - carry flag
		; R - result (normal order)
		(diff (lambda (a b c r)
			(cond ((null? a)
					(cond
						; A=() ; B=()
						((null? b)
							(cond
								; carry not set? OK
								((eq? c 0-digit) r)
								; A<B is undefined
								(#t (bottom
									'difference-negative))))
						(#t (bottom 'difference-negative))))

			; A\=() ; B=() ; see above
			((null? b)
				(cond
					; carry not set?
					((eq? c 0-digit)
						; prepend rest of A
						(append (reverse a) r))
					; else subtract borrow
					(#t (diff a (integer->list 1) 0-digit r))))

			; advance to next pair of digits
			(#t
				(diff (cdr a) (cdr b)
					; compute borrow
					(cdr (ddiff (car a) (car b) c 0-digit))
					; prepend result digit
					(cons (car (ddiff (car a)
						(car b) c 0-digit))
						r)))))))

		(normalize
			(list->integer (diff
				(reverse (integer->list a))
				(reverse (integer->list b))
				0-digit '()) #t)))))

