; SketchyLISP Library
; Copyright (C) 2005 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; normalize

; ---conformance---
; Sketchy Core

; ---purpose---
; Normalize a number by removing leading zeroes and plus signs.

; ---args---
; X - number

; ---keywords---
; NORMALIZE function, normalize, normal form, canonical number
; GCD, reducing, numbers, digits

; ---see-also---
; digits, n-, ndivide

; ---example---
; :l lib/list.l
; (normalize +000511) => 511

(define normalize #t)

:c lib/digits.l 0-digit
:c lib/caar.l cdar
:c lib/caaar.l caadr cdadr
:c lib/gcd.l
:c lib/equal.l =
:c lib/zerop.l zero?
:c lib/quotient.l
:c lib/minus.l -

; ---code---
(define normalize (lambda (x)
	(letrec
		; remove leading zeroes from natural number
		((skip0 (lambda (x)
			(cond ((null? (cdr x)) x)
				((eq? (car x) 0-digit)
					(skip0 (cdr x)))
				(#t x))))
		(norm (lambda (x)
			(cond ((eq? (car x) '+)
					(skip0 (cdr x)))
				((eq? (car x) '-)
					(cons '- (skip0 (cdr x))))
				(#t (skip0 x))))))
		(list->integer (norm (integer->list x)) #t))))

