; SketchyLISP Library
; Copyright (C) 2005 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; n+

; ---conformance---
; Sketchy Core

; ---purpose---
; Add two natural numbers.

; ---args---
; A - natural number
; B - natural number

; ---keywords---
; N+ function, plus, sum, addition, arithmetics, natural

; ---see-also---
; digits, n-, nquotient, nremainder, n*, +

; ---example---
; (n+ 5 7) => 12

(define nplus #t)

:c lib/digits.l 0-digit 1-digit
:c lib/reverse.l
:c lib/pred.l
:c lib/succ.l

; ---code---
(define n+ (lambda (a b)
	(letrec
	
		; Compute the sum of two digits and a carry flag.
		; X Y - digits
		; CI CO - carry in, out
		; RESULT - (sum . carry out)
		((dsum (lambda (x y ci co)
			(cond ((eq? x 0-digit)
					(cond ((eq? ci 1-digit)
							; add carry flag
							(dsum 1-digit y 0-digit co))
						(#t (cons y co))))
				; handle overflow
				((null? (succ y))
					(dsum (pred x) 0-digit ci 1-digit))
				(#t (dsum (pred x) (succ y) ci co)))))

		; Add two lists of decimal digits forming reverse
		; decimal numbers. The result is in normal order.
		; A B - reverse lists of digits
		; C - carry flag
		; R - result (normal order)
		(add (lambda (a b c r)
			(cond ((null? a)
					(cond
						; A=() ; B=()
						((null? b)
							(cond
								; carry not set? return R
								((eq? c 0-digit) r)	
								; else prepend overflow
								(#t (cons 1-digit r))))
						; B\=(): keep adding leading zeroes
						(#t (add '() (cdr b)
							(cdr (dsum 0-digit (car b) c 0-digit))
							(cons
								(car (dsum 0-digit (car b) c 0-digit))
								r)))))
			((null? b)
				; B=() ; A\=()
				(add (cdr a) '()
					(cdr (dsum (car a) 0-digit c 0-digit))
					(cons (car (dsum (car a) 0-digit c 0-digit)) r)))
			; default: advance to next two digits
			(#t (add (cdr a) (cdr b)
				(cdr (dsum (car a) (car b) c 0-digit))
				(cons (car (dsum (car a) (car b) c 0-digit)) r)))))))

		(list->integer
			(add (reverse (integer->list a))
				(reverse (integer->list b))
				0-digit '()) #t))))

