; SketchyLISP Library
; Copyright (C) 2005 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; +

; ---conformance---
; R5RS

; ---purpose---
; Add numbers.

; ---args---
; A - number
; B... - numbers

; ---keywords---
; + function, plus, sum, addition, arithmetics

; ---see-also---
; digits, -, quotient, remainder, *, n+

; ---example---
; (+ 5 7 9) => 21

(define plus #t)

:c lib/digits.l
:c lib/abs.l
:c lib/ngreater.l n>
:c lib/logand.l
:c lib/minus.l -
:c lib/nplus.l n+
:c lib/integer.l
:c lib/iterate.l
:c lib/negativep.l negative?
:c lib/non-negativep.l non-negative?
:c lib/negate.l

; ---code---
(define + (lambda (a . b)
	(letrec

	((_iplus (lambda (a b)
		; A>=0, B>=0: A+B => |A|+|B|
		(cond ((logand (non-negative? a) (non-negative? b))
				(n+ (abs a) (abs b)))
			; A>=0, B<0
			((logand (non-negative? a) (negative? b))
				; |A|>|B| => A-|B|
				(cond ((n> (abs a) (abs b)) (- a (abs b)))
					; |A|<=|B| => -(|B|-A)
					(#t (negate (- (abs b) a)))))
			; A<0, B>=0
			((logand (negative? a) (non-negative? b))
				; |A|>|B| => -(|A|-B)
				(cond ((n> (abs a) (abs b))
						(negate (- (abs a) b)))
					; |A|<=|B| => B-|A|
					(#t (- b (abs a)))))
			; A<0, B<0 => -(|A|+|B|)
			(#t (negate (n+ (abs a) (abs b)))))))

		(i+ (lambda (a b) (_iplus (integer a) (integer b)))))

		(cond ((null? b) a)
			(#t (iterate i+ (cons a b)))))))

