; SketchyLISP Library
; Copyright (C) 2005 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; sqrt

; ---conformance---
; R5RS
; Result is an integer.

; ---purpose---
; Compute the square root of a natural number.
; This function returns a natural result, which
; may be inexact.

; ---args---
; X - square of root to extract

; ---keywords---
; SQRT function, root, square, square root, arithmetics

; ---see-also---
; digits, expt, gcd

; ---example---
; (sqrt 144) => 12

(define sqrt #t)

:c lib/list.l
:c lib/equal.l =
:c lib/negativep.l negative?
:c lib/ntimes.l n*
:c lib/nquotient.l
:c lib/nplus.l n+
:c lib/ngreater.l n>
:c lib/nlteq.l n<=

; ---model---
; (define sqrt (lambda (square)
;	(letrec ((sqr (lambda (x last)
;			(cond ((logand (n<= (n* x x) square)
;						(n> (n* (n+ x 1) (n+ x 1)) square))
;					x)
;				(#t (sqr
;					(nquotient
;						(n+ x (nquotient square x))
;						2) x))))))
;		(sqr square 0))))

; ---code---
(define sqrt (lambda (square)
	(letrec ((sqr (lambda (x last)
			(cond ((= last x) x)
				((= last (n+ x 1))
					(cond ((n> (n* x x) square) (- x 1))
						(#t x)))
				(#t (sqr
					(nquotient
						(n+ x (nquotient square x))
						2) x))))))
		(cond ((negative? square)
				(bottom (list 'sqrt square)))
			(#t (sqr square 0))))))

