; SketchyLISP Library
; Copyright (C) 2005 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; *

; ---conformance---
; R5RS

; ---purpose---
; Multiply numbers.

; ---args---
; A - number
; B... - numbers

; ---keywords---
; * function, times, multiply, multiplication, product
; arithmetics

; ---see-also---
; digits, +, -, quotient, remainder, n*

; ---example---
; (* 2 3 4) => 24

(define times #t)

:c lib/digits.l
:c lib/abs.l
:c lib/minus.l -
:c lib/integer.l
:c lib/ntimes.l n*
:c lib/zerop.l zero?
:c lib/negate.l
:c lib/iterate.l

; ---code---
(define * (lambda (a . b)
	(letrec

		((_itimes (lambda (a b)
			(cond
				; avoid leading zeroes in result
				((zero? a) 0)
				; handle signs
				((eq? (negative? a) (negative? b))
					(n* (abs a) (abs b)))
				(#t (negate (n* (abs a) (abs b)))))))

		(i* (lambda (a b) (_itimes (integer a) (integer b)))))

		(cond ((null? b) a)
			(#t (iterate i* (cons a b)))))))

