; SketchyLISP Library
; Copyright (C) 2005 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; union

; ---conformance---
; Sketchy Extension

; ---purpose---
; Compute the union of two sets.
; Sets are represented by lists
; of unique members.

; ---args---
; A - set
; B - set

; ---keywords---
; UNION function, union, sets, set theory

; ---see-also---
; intersection, member, remove, unique, any?

; ---example---
; (union '(a b c) '(b d)) => '(a b c d)

(define union #t)

:c lib/memberp.l member?
:c lib/append.l
:c lib/reverse.l
:c lib/unique.l

; ---model---
; (define union (lambda (a b)
;	(cond ((null? a) b)
;		((member? (car a) b) (union (cdr a) b))
;		(#t (cons (car a) (union (cdr a) b))))))

; ---model---
; (define union (lambda (a b)
;	(letrec ((uon (lambda (a b r)
;			(cond ((null? a) (append (reverse r) b))
;				((member? (car a) b) (uon (cdr a) b r))
;				(#t (uon (cdr a) b (cons (car a) r)))))))
;		(uon a b '()))))

; ---code---
(define union (lambda (a b)
	(unique (append a b))))

