; SketchyLISP Example Program
; Copyright (C) 2005 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---purpose---
; Evaluate arithmetic expressions.
; Evaluate all operations with numeric arguments
; (or subexpressions evaluating to numeric arguments)
; to their results. Reduce expressions with one variable
; argument using rules such as X+0 -> X and X^0 -> 1.
; Return the evaluated expression.
; --
; The following functions are recognized:
; +, -, *, QUOTIENT, EXPT
; --
; Variables are represented by single letters.

; ---keywords---
; EVALUATE function, evaluation, partial, math expressions
; arithmetic expressions, arithmetics

; ---args---
; X - expression

; ---example---
; (evaluate '(* x (expt 2 (+ 3 4)))) => '(* x 128)

:l lib/list.l
:l lib/memberp.l member?
:l lib/caar.l cadr.l
:l lib/caaar.l caddr.l
:l lib/lognot.l
:l lib/logand.l
:l lib/zerop.l zero?
:l lib/onep.l one?
:l lib/plus.l +
:l lib/minus.l -
:l lib/times.l *
:l lib/quotient.l
:l lib/expt.l

; ---code---
(define evaluate (lambda (x)
	(letrec

	; Evaluate (fn left-num right-num)
	((compute (lambda (op left right)
		(cond ((eq? op '+) (+ left right))
			((eq? op '-) (- left right))
			((eq? op '*) (* left right))
			((eq? op 'quotient) (quotient left right))
			((eq? op 'expt) (expt left right))
			(#t (bottom 'compute)))))

	(zero-p (lambda (x)
		(cond ((symbol? x) #f)
			(#t (zero? x)))))

	(one-p (lambda (x)
		(cond ((symbol? x) #f)
			(#t (one? x)))))

	(evl (lambda (x)
		(cond ((symbol? x) x)	; Pass thru symbols and numbers
			((number? x) x)
			; Evaluate or reduce binary functions
			((member? (car x) '(+ - * quotient expt))
				; First evaluate the arguments
				(letrec ((left (evl (cadr x)))
						(right (evl (caddr x))))
				(cond
					; Both arguments constant,
					; compute the result
					((logand (number? left) (number? right))
						(compute (car x) left right))
					((eq? (car x) '+)
						(cond ((zero-p right) left)		; X+0 => X
							((zero-p left) right)		; 0+X => X
							(#t (list (car x) left right))))
					((eq? (car x) '-)
						(cond ((zero-p right) left)		; X-0 => X
							(#t (list (car x) left right))))
					((eq? (car x) '*)
						(cond ((zero-p right) '0)		; X*0 => 0
							((zero-p left) '0)		; 0*X => 0
							((one-p left) right)		; X*1 => X
							((one-p right) left)		; 1*X => X
							(#t (list (car x) left right))))
					((eq? (car x) 'quotient)
						(cond ((zero-p right)			; X/0 => _|_
								(bottom 'divide-by-zero))
							((zero-p left) '0)		; 0/X => 0
							((one-p right) left)		; X/1 => X
							(#t (list (car x) left right))))
					((eq? (car x) 'expt)
						(cond ((zero-p right) '1)		; X^0 => 1
							((zero-p left) '0)		; 0^X => 0
							((one-p left) '1)			; 1^X => 1
							((one-p right) left)		; X^1 => X
							(#t (list (car x) left right))))
					; Unknown operator, pass thru
					(#t (bottom 'this-cannot-happen)))))
			(#t (bottom (list 'unknown-operator: x)))))))

	(evl x))))

